From d2fd8a73e97800050d6fce22ca3815b159cc0efe Mon Sep 17 00:00:00 2001
From: Paul Eggert <eggert@cs.ucla.edu>
Date: Mon, 23 May 2016 16:53:34 -0700
Subject: [PATCH] Fix vc-diff problems with UTF-16

Problem with UTF-16 reported by Uwe Brauer (Bug#23595).
There are similar problems with EBCDIC or with other coding
systems differing greatly from ASCII.  Also, the heuristic should
be more conservative when dealing with multiple files that have
incompatible coding systems.
* lisp/vc/vc.el (vc-diff-internal): Require all the files to have
the same coding system, and for that coding system to be
compatible-enough with ASCII so that messages like "Binary files
differ" are not misdecoded.
---
 lisp/vc/vc.el | 25 +++++++++++++++++++++----
 1 file changed, 21 insertions(+), 4 deletions(-)

diff --git a/lisp/vc/vc.el b/lisp/vc/vc.el
index 25b41e3..c69ff23 100644
--- a/lisp/vc/vc.el
+++ b/lisp/vc/vc.el
@@ -1684,11 +1684,28 @@ vc-diff-internal
                          (format "No changes between %s and %s"
                                  (or rev1 "working revision")
                                  (or rev2 "workfile"))))
-	 ;; Set coding system based on the first file.  It's a kluge,
-	 ;; but the only way to set it for each file included would
-	 ;; be to call the back end separately for each file.
 	 (coding-system-for-read
-	  (if files (vc-coding-system-for-diff (car files)) 'undecided)))
+	  (or coding-system-for-read
+	      (when files
+		;; If all files use the same coding system, default to that.
+		;; It's a kludge, but the only way to set it for each file would
+		;; be to call the back end separately for each file.
+		(let ((coding (vc-coding-system-for-diff (car files))))
+		  (let ((fs (cdr files)))
+		    (while (and fs coding)
+		      (unless (eq coding (vc-coding-system-for-diff (car fs)))
+			(setq coding nil))
+		      (setq fs (cdr fs))))
+		  ;; If the files' coding system is UTF-16, diff likely
+		  ;; outputs something like "Binary files differ" in ASCII,
+		  ;; which is misdecoded as UTF-16.  So use the files' coding
+		  ;; system only if it is compatible-enough with ASCII.
+		  (when (and coding
+			     (let ((samp "Binary files differ"))
+			       (string-equal samp (decode-coding-string
+						   samp coding t))))
+		    last-coding-system-used))))))
+
     ;; On MS-Windows and MS-DOS, Diff is likely to produce DOS-style
     ;; EOLs, which will look ugly if (car files) happens to have Unix
     ;; EOLs.
-- 
2.5.5

